/**
 * This class was created by <Vazkii>. It's distributed as
 * part of the Botania Mod. Get the Source Code in github:
 * https://github.com/Vazkii/Botania
 * 
 * Botania is Open Source and distributed under the
 * Botania License: http://botaniamod.net/license.php
 * 
 * File Created @ [Jan 14, 2014, 6:46:59 PM (GMT)]
 */
package xyz.brassgoggledcoders.boilerplate.lib.client.manual;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import net.minecraft.util.text.TextFormatting;
import net.minecraft.util.text.translation.I18n;
import org.lwjgl.input.Mouse;

import net.minecraft.client.gui.Gui;
import net.minecraft.client.gui.GuiButton;
import net.minecraft.client.gui.GuiTextField;
import net.minecraft.client.renderer.GlStateManager;
import net.minecraft.init.Items;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import xyz.brassgoggledcoders.boilerplate.lib.client.manual.button.GuiButtonBack;
import xyz.brassgoggledcoders.boilerplate.lib.client.manual.button.GuiButtonInvisible;
import xyz.brassgoggledcoders.boilerplate.lib.client.manual.button.GuiButtonPage;
import xyz.brassgoggledcoders.boilerplate.lib.common.manual.BotaniaAPI;
import xyz.brassgoggledcoders.boilerplate.lib.common.manual.ILexicon;
import xyz.brassgoggledcoders.boilerplate.lib.common.manual.LexiconCategory;
import xyz.brassgoggledcoders.boilerplate.lib.common.manual.LexiconEntry;

public class GuiLexiconIndex extends GuiLexicon implements IParented {

	private static final String TAG_CATEGORY = "category";
	private static final String TAG_PAGE = "page";

	LexiconCategory category;
	String title;
	int page = 0;

	int tutPage = -1;

	GuiButton leftButton, rightButton, backButton;
	GuiLexicon parent;
	GuiTextField searchField;

	GuiButton currentButton;
	LexiconEntry currentEntry;
	float infoTime;

	List<LexiconEntry> entriesToDisplay = new ArrayList<LexiconEntry>();

	public GuiLexiconIndex() {
		parent = new GuiLexicon();
	}

	public GuiLexiconIndex(LexiconCategory category) {
		this.category = category;
		parent = new GuiLexicon();
		setTitle();
	}

	public void setTitle() {
		title = I18n.func_74838_a(category == null ? "botaniamisc.lexiconIndex" : category.getUnlocalizedName());
	}

	@Override
	boolean isMainPage() {
		return false;
	}

	@Override
	String getTitle() {
		return title;
	}

	@Override
	boolean isIndex() {
		return true;
	}

	@Override
	boolean isCategoryIndex() {
		return false;
	}

	@Override
	public void onInitGui() {
		super.onInitGui();

		if(!GuiLexicon.isValidLexiconGui(this))	{
			currentOpenLexicon = new GuiLexicon();
			field_146297_k.func_147108_a(currentOpenLexicon);
			return;
		}

		field_146292_n.add(backButton = new GuiButtonBack(12, left + guiWidth / 2 - 8, top + guiHeight + 2));
		field_146292_n.add(leftButton = new GuiButtonPage(13, left, top + guiHeight - 10, false));
		field_146292_n.add(rightButton = new GuiButtonPage(14, left + guiWidth - 18, top + guiHeight - 10, true));

		searchField = new GuiTextField(15, field_146289_q, left + guiWidth / 2 + 28, top + guiHeight + 6, 200, 10);
		searchField.func_146205_d(false);
		searchField.func_146195_b(true);
		searchField.func_146185_a(false);

		updateAll();
	}

	void updateAll() {
		buildEntries();
		updatePageButtons();
		populateIndex();
	}

	void buildEntries() {
		entriesToDisplay.clear();
		ILexicon lex = (ILexicon) stackUsed.func_77973_b();
		for(LexiconEntry entry : category == null ? BotaniaAPI.getAllEntries() : category.entries) {
			if(entry.isVisible() && matchesSearch(entry))
				entriesToDisplay.add(entry);
		}
		Collections.sort(entriesToDisplay);
	}
	
	boolean matchesSearch(LexiconEntry e) {
		String search = searchField.func_146179_b().trim();
		if(search.isEmpty())
			return true;
		
		search = search.toLowerCase();
		if(I18n.func_74838_a(e.getUnlocalizedName()).toLowerCase().contains(search))
			return true;
		
		for(ItemStack stack : e.getDisplayedRecipes()) {
			String stackName = stack.func_82833_r().toLowerCase().trim();
			if(stackName.contains(search))
				return true;
		}
				
		return false;
	}

	@Override
	void populateIndex() {
		for(int i = page * 12; i < (page + 1) * 12; i++) {
			GuiButtonInvisible button = (GuiButtonInvisible) field_146292_n.get(i - page * 12);
			LexiconEntry entry = i >= entriesToDisplay.size() ? null : entriesToDisplay.get(i);
			if(entry != null) {
				button.field_146126_j = (entry.isPriority() ? TextFormatting.ITALIC : "") + I18n.func_74838_a(entry.getUnlocalizedName());
				button.displayStack = entry.getIcon();
				
			} else button.field_146126_j = "";
		}
	}

	public void setHoveredButton(GuiButtonInvisible b) {
		if(b == null)
			currentEntry = null;
		else currentEntry = entriesToDisplay.get(b.field_146127_k + page * 12);
		currentButton = b;
	}

	@Override
	public void func_73863_a(int par1, int par2, float par3) {
		super.func_73863_a(par1, par2, par3);

		if(!searchField.func_146179_b().isEmpty()) {
			field_146297_k.field_71446_o.func_110577_a(texture);
			GlStateManager.func_179131_c(1F, 1F, 1F, 1F);
			func_73729_b(left + 134, top + guiHeight - 26, 86, 180, 12, 12);
			
			if(entriesToDisplay.size() == 1) {
				boolean unicode = field_146297_k.field_71466_p.func_82883_a();
				field_146297_k.field_71466_p.func_78264_a(true);
				String s = I18n.func_74838_a("botaniamisc.enterToView");
				field_146297_k.field_71466_p.func_78276_b(s, left + guiWidth / 2 - field_146297_k.field_71466_p.func_78256_a(s) / 2, top + 30, 0x666666);
				field_146297_k.field_71466_p.func_78264_a(unicode);
			}
		} else {
			boolean unicode = field_146297_k.field_71466_p.func_82883_a();
			field_146297_k.field_71466_p.func_78264_a(true);
			String s = I18n.func_74838_a("botaniamisc.typeToSearch");
			field_146297_k.field_71466_p.func_78276_b(s, left + 120 - field_146297_k.field_71466_p.func_78256_a(s), top + guiHeight - 18, 0x666666);
			field_146297_k.field_71466_p.func_78264_a(unicode);
		}

		float animationTime = 4F;
		if(func_146272_n()) {
			if(currentButton != null)
				infoTime = Math.min(animationTime, infoTime + timeDelta);
		} else {
			infoTime = Math.max(0, infoTime - timeDelta);

			if(currentButton != null && infoTime == 0) {
				int x = par1 + 10;
				int y = par2;

				x = currentButton.field_146128_h - 20;
				y = currentButton.field_146129_i;

				field_146297_k.field_71466_p.func_175063_a("?", x, y, 0xFFFFFF);
				GlStateManager.func_179152_a(0.5F, 0.5F, 1F);
				field_146297_k.field_71466_p.func_175063_a(TextFormatting.BOLD + "Shift", x * 2 - 6, y * 2 + 20, 0xFFFFFF);
				GlStateManager.func_179152_a(2F, 2F, 1F);
			}
		}

		if(currentButton != null && infoTime > 0) {
			float fract = infoTime / animationTime;

			int x = currentButton.field_146128_h;
			int y = currentButton.field_146129_i;
			String s = I18n.func_74838_a(currentEntry.getTagline());
			boolean unicode = field_146297_k.field_71466_p.func_82883_a();
			field_146297_k.field_71466_p.func_78264_a(true);
			int width = field_146297_k.field_71466_p.func_78256_a(s);

			GlStateManager.func_179094_E();
			GlStateManager.func_179109_b(x, y, 0);
			GlStateManager.func_179152_a(fract, 1F, 1F);
			Gui.func_73734_a(12, -30, width + 20, -2, 0x44000000);
			Gui.func_73734_a(10, -32, width + 22, -2, 0x44000000);

			field_146297_k.field_71466_p.func_78264_a(unicode);

			net.minecraft.client.renderer.RenderHelper.func_74520_c();
			GlStateManager.func_179091_B();
			ItemStack paper = new ItemStack(Items.field_151121_aF, currentEntry.pages.size());

			field_146297_k.func_175599_af().func_180450_b(paper, 14, -28);
			field_146297_k.func_175599_af().func_180453_a(field_146297_k.field_71466_p, paper, 14, -28, "");
			List<ItemStack> stacks = currentEntry.getDisplayedRecipes();

			if(stacks.size() > 0) {
				int spaceForEach = Math.min(18, (width - 30) / stacks.size());
				for(int i = 0; i < stacks.size(); i++) {
					ItemStack stack = stacks.get(i);
					field_146297_k.func_175599_af().func_180450_b(stack, 38 + spaceForEach * i, -28);
				}
			}

			net.minecraft.client.renderer.RenderHelper.func_74518_a();

			GlStateManager.func_179121_F();
		}

		setHoveredButton(null);
	}

	@Override
	protected void func_146284_a(GuiButton par1GuiButton) {
			switch(par1GuiButton.field_146127_k) {
			case 12 :
				field_146297_k.func_147108_a(parent);
				break;
			case 13 :
				page--;
				updatePageButtons();
				populateIndex();
				break;
			case 14 :
				page++;
				updatePageButtons();
				populateIndex();
				break;
			default :
					int index = par1GuiButton.field_146127_k + page * 12;
					openEntry(index);
			}
	}

	void openEntry(int index) {
		if(index >= entriesToDisplay.size())
			return;

		LexiconEntry entry = entriesToDisplay.get(index);
		field_146297_k.func_147108_a(new GuiLexiconEntry(entry, this));
	}

	public void updatePageButtons() {
		leftButton.field_146124_l = page != 0;
		rightButton.field_146124_l = page < (entriesToDisplay.size() - 1) / 12;
	}

	@Override
	public void setParent(GuiLexicon gui) {
		parent = gui;
	}

	int fx = 0;
	boolean swiped = false;

	@Override
	protected void func_146273_a(int x, int y, int button, long time) {
		if(button == 0 && Math.abs(x - fx) > 100 && field_146297_k.field_71474_y.field_85185_A && !swiped) {
			double swipe = (x - fx) / Math.max(1, (double) time);
			if(swipe < 0.5) {
				nextPage();
				swiped = true;
			} else if(swipe > 0.5) {
				prevPage();
				swiped = true;
			}
		}
	}

	@Override
	protected void func_73864_a(int par1, int par2, int par3) throws IOException {
		super.func_73864_a(par1, par2, par3);

		searchField.func_146192_a(par1, par2, par3);
		fx = par1;
		switch(par3) {
		case 1:
			back();
			break;
		case 3:
			nextPage();
			break;
		case 4:
			prevPage();
			break;
		}
	}

	@Override
	public void func_146274_d() throws IOException {
		super.func_146274_d();

		if(Mouse.getEventButton() == 0)
			swiped = false;

		int w = Mouse.getEventDWheel();
		if(w < 0)
			nextPage();
		else if(w > 0)
			prevPage();
	}

	@Override
	boolean closeScreenOnInvKey() {
		return false;
	}

	@Override
	protected void func_73869_a(char par1, int par2) throws IOException {
		if(par2 == 203 || par2 == 200 || par2 == 201) // Left, Up, Page Up
			prevPage();
		else if(par2 == 205 || par2 == 208 || par2 == 209) // Right, Down Page Down
			nextPage();
		else if(par2 == 14 && searchField.func_146179_b().isEmpty()) // Backspace
			back();
		else if(par2 == 199) { // Home
			field_146297_k.func_147108_a(new GuiLexicon());
		} else if(par2 == 28 && entriesToDisplay.size() == 1) // Enter
			openEntry(0);

		super.func_73869_a(par1, par2);
	}

	void back() {
		if(backButton.field_146124_l) {
			func_146284_a(backButton);
			backButton.func_146113_a(field_146297_k.func_147118_V());
		}
	}

	void nextPage() {
		if(rightButton.field_146124_l) {
			func_146284_a(rightButton);
			rightButton.func_146113_a(field_146297_k.func_147118_V());
		}
	}

	void prevPage() {
		if(leftButton.field_146124_l) {
			func_146284_a(leftButton);
			leftButton.func_146113_a(field_146297_k.func_147118_V());
		}
	}

	@Override
	public void serialize(NBTTagCompound cmp) {
		super.serialize(cmp);
		cmp.func_74778_a(TAG_CATEGORY, category == null ? "" : category.getUnlocalizedName());
		cmp.func_74768_a(TAG_PAGE, page);
	}

	@Override
	public void load(NBTTagCompound cmp) {
		super.load(cmp);
		String categoryStr = cmp.func_74779_i(TAG_CATEGORY);
		if(categoryStr.isEmpty())
			category = null;
		else for(LexiconCategory cat : BotaniaAPI.getAllCategories())
			if(cat.getUnlocalizedName().equals(categoryStr)) {
				category = cat;
				break;
			}
		page = cmp.func_74762_e(TAG_PAGE);
		setTitle();
	}

	@Override
	public GuiLexicon copy() {
		GuiLexiconIndex gui = new GuiLexiconIndex(category);
		gui.page = page;
		gui.setTitle();
		return gui;
	}

	@Override
	public String getNotesKey() {
		return "category_" + (category == null ? "lexindex" : category.unlocalizedName);
	}
}

