/**
 * This class was created by <Vazkii>. It's distributed as
 * part of the Botania Mod. Get the Source Code in github:
 * https://github.com/Vazkii/Botania
 * 
 * Botania is Open Source and distributed under the
 * Botania License: http://botaniamod.net/license.php
 * 
 * File Created @ [Jan 19, 2014, 4:58:19 PM (GMT)]
 */
package xyz.brassgoggledcoders.boilerplate.lib.common.manual.page;

import net.minecraft.client.Minecraft;
import net.minecraft.client.gui.GuiScreen;
import net.minecraft.client.renderer.GlStateManager;
import net.minecraft.client.renderer.texture.TextureManager;
import net.minecraft.item.ItemStack;
import net.minecraft.item.crafting.IRecipe;
import net.minecraft.item.crafting.ShapedRecipes;
import net.minecraft.item.crafting.ShapelessRecipes;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.text.translation.I18n;
import net.minecraftforge.fml.relauncher.ReflectionHelper;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import net.minecraftforge.oredict.ShapedOreRecipe;
import net.minecraftforge.oredict.ShapelessOreRecipe;
import org.lwjgl.opengl.GL11;
import xyz.brassgoggledcoders.boilerplate.lib.client.manual.RenderHelper;
import xyz.brassgoggledcoders.boilerplate.lib.common.manual.IGuiLexiconEntry;
import xyz.brassgoggledcoders.boilerplate.lib.common.manual.LexiconEntry;
import xyz.brassgoggledcoders.boilerplate.lib.common.manual.LexiconRecipeMappings;
import xyz.brassgoggledcoders.boilerplate.lib.common.utils.Utils;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class PageCraftingRecipe extends PageRecipe {

	private static final ResourceLocation craftingOverlay = new ResourceLocation(Utils.getCurrentMod().getPrefix() + "textures/gui/manual/craftingOverlay.png");

	List<IRecipe> recipes;
	int ticksElapsed = 0;
	int recipeAt = 0;

	boolean oreDictRecipe, shapelessRecipe;

	public PageCraftingRecipe(String unlocalizedName, List<IRecipe> recipes) {
		super(unlocalizedName);
		this.recipes = recipes;
	}

	public PageCraftingRecipe(String unlocalizedName, IRecipe recipe) {
		this(unlocalizedName, Collections.singletonList(recipe));
	}

	@Override
	public void onPageAdded(LexiconEntry entry, int index) {
		for(IRecipe recipe : recipes)
			LexiconRecipeMappings.map(recipe.func_77571_b(), entry, index);
	}

	@Override
	@SideOnly(Side.CLIENT)
	public void renderRecipe(IGuiLexiconEntry gui, int mx, int my) {
		oreDictRecipe = shapelessRecipe = false;

		IRecipe recipe = recipes.get(recipeAt);
		renderCraftingRecipe(gui, recipe);

		TextureManager render = Minecraft.func_71410_x().field_71446_o;
		render.func_110577_a(craftingOverlay);

		GlStateManager.func_179147_l();
		GlStateManager.func_179112_b(GL11.GL_SRC_ALPHA, GL11.GL_ONE_MINUS_SRC_ALPHA);
		GlStateManager.func_179131_c(1F, 1F, 1F, 1F);
		((GuiScreen) gui).func_73729_b(gui.getLeft(), gui.getTop(), 0, 0, gui.getWidth(), gui.getHeight());

		int iconX = gui.getLeft() + 115;
		int iconY = gui.getTop() + 12;

		GlStateManager.func_179147_l();
		GlStateManager.func_179112_b(GL11.GL_SRC_ALPHA, GL11.GL_ONE_MINUS_SRC_ALPHA);

		if(shapelessRecipe) {
			((GuiScreen) gui).func_73729_b(iconX, iconY, 240, 0, 16, 16);

			if(mx >= iconX && my >= iconY && mx < iconX + 16 && my < iconY + 16)
				RenderHelper.renderTooltip(mx, my, Collections.singletonList(I18n.func_74838_a("botaniamisc.shapeless")));

			iconY += 20;
		}

		render.func_110577_a(craftingOverlay);
		GlStateManager.func_179147_l();

		if(oreDictRecipe) {
			((GuiScreen) gui).func_73729_b(iconX, iconY, 240, 16, 16, 16);

			if(mx >= iconX && my >= iconY && mx < iconX + 16 && my < iconY + 16)
				RenderHelper.renderTooltip(mx, my, Collections.singletonList(I18n.func_74838_a("botaniamisc.oredict")));
		}
		GlStateManager.func_179084_k();
	}

	@Override
	@SideOnly(Side.CLIENT)
	public void updateScreen() {
		if(GuiScreen.func_146272_n())
			return;

		if(ticksElapsed % 20 == 0) {
			recipeAt++;

			if(recipeAt == recipes.size())
				recipeAt = 0;
		}
		++ticksElapsed;
	}

	@SideOnly(Side.CLIENT)
	public void renderCraftingRecipe(IGuiLexiconEntry gui, IRecipe recipe) {
		if(recipe instanceof ShapedRecipes) {
			ShapedRecipes shaped = (ShapedRecipes)recipe;

			for(int y = 0; y < shaped.field_77577_c; y++)
				for(int x = 0; x < shaped.field_77576_b; x++)
					renderItemAtGridPos(gui, 1 + x, 1 + y, shaped.field_77574_d[y * shaped.field_77576_b + x], true);
		} else if(recipe instanceof ShapedOreRecipe) {
			ShapedOreRecipe shaped = (ShapedOreRecipe) recipe;
			int width = ReflectionHelper.getPrivateValue(ShapedOreRecipe.class, shaped, 4);
			int height = ReflectionHelper.getPrivateValue(ShapedOreRecipe.class, shaped, 5);

			for(int y = 0; y < height; y++)
				for(int x = 0; x < width; x++) {
					Object input = shaped.getInput()[y * width + x];
					if(input != null)
						renderItemAtGridPos(gui, 1 + x, 1 + y, input instanceof ItemStack ? (ItemStack) input : ((List<ItemStack>) input).get(0), true);
				}

			oreDictRecipe = true;
		} else if(recipe instanceof ShapelessRecipes) {
			ShapelessRecipes shapeless = (ShapelessRecipes) recipe;

			drawGrid : {
				for(int y = 0; y < 3; y++)
					for(int x = 0; x < 3; x++) {
						int index = y * 3 + x;

						if(index >= shapeless.field_77579_b.size())
							break drawGrid;

						renderItemAtGridPos(gui, 1 + x, 1 + y, shapeless.field_77579_b.get(index), true);
					}
			}

			shapelessRecipe = true;
		} else if(recipe instanceof ShapelessOreRecipe) {
			ShapelessOreRecipe shapeless = (ShapelessOreRecipe) recipe;

			drawGrid : {
				for(int y = 0; y < 3; y++)
					for(int x = 0; x < 3; x++) {
						int index = y * 3 + x;

						if(index >= shapeless.func_77570_a())
							break drawGrid;

						Object input = shapeless.getInput().get(index);
						if(input != null)
							renderItemAtGridPos(gui, 1 + x, 1 + y, input instanceof ItemStack ? (ItemStack) input : ((List<ItemStack>) input).get(0), true);
					}
			}

			shapelessRecipe = true;
			oreDictRecipe = true;
		}

		renderItemAtGridPos(gui, 2, 0, recipe.func_77571_b(), false);
	}

	@Override
	public List<ItemStack> getDisplayedRecipes() {
		ArrayList<ItemStack> list = new ArrayList<>();
		for(IRecipe r : recipes)
			list.add(r.func_77571_b());

		return list;
	}
}
